@@ -3,7 +3,8 @@ __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 '''Dialog to edit metadata in bulk'''
 
-import re, os
+import re
+from io import BytesIO
 from collections import namedtuple, defaultdict
 from threading import Thread
 
@@ -11,8 +12,12 @@ from PyQt5.Qt import Qt, QDialog, QGridLayout, QVBoxLayout, QFont, QLabel, \
                      pyqtSignal, QDialogButtonBox, QInputDialog, QLineEdit, \
                      QDateTime, QCompleter, QCoreApplication, QSize
 
+from calibre import prints
+from calibre.ebooks.metadata.opf2 import OPF
+from calibre.constants import DEBUG
 from calibre.gui2.dialogs.metadata_bulk_ui import Ui_MetadataBulkDialog
 from calibre.gui2.dialogs.tag_editor import TagEditor
+from calibre.gui2.dialogs.template_line_editor import TemplateLineEditor
 from calibre.ebooks.metadata import string_to_authors, authors_to_string, title_sort
 from calibre.ebooks.metadata.book.formatter import SafeFormat
 from calibre.gui2.custom_column_widgets import populate_metadata_page
@@ -24,42 +29,15 @@ from calibre.utils.config import dynamic, JSONConfig
 from calibre.utils.titlecase import titlecase
 from calibre.utils.icu import sort_key, capitalize
 from calibre.utils.config import prefs, tweaks
-from calibre.utils.imghdr import identify
 from calibre.utils.date import qt_to_dt
 from calibre.db import _get_next_series_num_for_list
 
 
-def get_cover_data(stream, ext):  # {{{
-    from calibre.ebooks.metadata.meta import get_metadata
-    old = prefs['read_file_metadata']
-    if not old:
-        prefs['read_file_metadata'] = True
-    cdata = area = None
-
-    try:
-        with stream:
-            mi = get_metadata(stream, ext)
-        if mi.cover and os.access(mi.cover, os.R_OK):
-            cdata = open(mi.cover).read()
-        elif mi.cover_data[1] is not None:
-            cdata = mi.cover_data[1]
-        if cdata:
-            fmt, width, height = identify(cdata)
-            area = width*height
-    except:
-        cdata = area = None
-
-    if old != prefs['read_file_metadata']:
-        prefs['read_file_metadata'] = old
-
-    return cdata, area
-# }}}
-
 Settings = namedtuple('Settings',
-    'remove_all remove add au aus do_aus rating pub do_series do_autonumber do_remove_format '
-    'remove_format do_swap_ta do_remove_conv do_auto_author series do_series_restart series_start_value series_increment '
+    'remove_all remove add au aus do_aus rating pub do_series do_autonumber '
+    'do_swap_ta do_remove_conv do_auto_author series do_series_restart series_start_value series_increment '
     'do_title_case cover_action clear_series clear_pub pubdate adddate do_title_sort languages clear_languages '
-    'restore_original comments generate_cover_settings')
+    'restore_original comments generate_cover_settings read_file_metadata')
 
 null = object()
 
@@ -132,9 +110,49 @@ class MyBlockingBusy(QDialog):  # {{{
 
         self.all_done.emit()
 
+    def read_file_metadata(self, args):
+        from calibre.utils.ipc.simple_worker import offload_worker
+        db = self.db.new_api
+        worker = offload_worker()
+        try:
+            for book_id in self.ids:
+                fmts = db.formats(book_id, verify_formats=False)
+                paths = filter(None, [db.format_abspath(book_id, fmt) for fmt in fmts])
+                if paths:
+                    ret = worker(
+                        'calibre.ebooks.metadata.worker', 'read_metadata_bulk',
+                        args.read_file_metadata, args.cover_action == 'fromfmt', paths)
+                    if ret['tb'] is not None:
+                        prints(ret['tb'])
+                    else:
+                        ans = ret['result']
+                        opf, cdata = ans['opf'], ans['cdata']
+                        if opf is not None:
+                            try:
+                                mi = OPF(BytesIO(opf), populate_spine=False, try_to_guess_cover=False).to_book_metadata()
+                            except Exception:
+                                import traceback
+                                traceback.print_exc()
+                            else:
+                                db.set_metadata(book_id, mi, allow_case_change=True)
+                        if cdata is not None:
+                            db.set_cover({book_id: cdata})
+        finally:
+            worker.shutdown()
+
     def do_all(self):
         cache = self.db.new_api
         args = self.args
+        from_file = args.cover_action == 'fromfmt' or args.read_file_metadata
+        if from_file:
+            old = prefs['read_file_metadata']
+            if not old:
+                prefs['read_file_metadata'] = True
+            try:
+                self.read_file_metadata(args)
+            finally:
+                if old != prefs['read_file_metadata']:
+                    prefs['read_file_metadata'] = old
 
         # Title and authors
         if args.do_swap_ta:
@@ -188,21 +206,6 @@ class MyBlockingBusy(QDialog):  # {{{
                 mi = self.db.get_metadata(book_id, index_is_id=True)
                 cdata = generate_cover(mi, prefs=args.generate_cover_settings)
                 cache.set_cover({book_id:cdata})
-        elif args.cover_action == 'fromfmt':
-            for book_id in self.ids:
-                fmts = cache.formats(book_id, verify_formats=False)
-                if fmts:
-                    covers = []
-                    for fmt in fmts:
-                        fmtf = cache.format(book_id, fmt, as_file=True)
-                        if fmtf is None:
-                            continue
-                        cdata, area = get_cover_data(fmtf, fmt)
-                        if cdata:
-                            covers.append((cdata, area))
-                    covers.sort(key=lambda x: x[1])
-                    if covers:
-                        cache.set_cover({book_id:covers[-1][0]})
         elif args.cover_action == 'trim':
             from calibre.utils.img import remove_borders_from_image, image_to_data, image_from_data
             for book_id in self.ids:
@@ -222,10 +225,6 @@ class MyBlockingBusy(QDialog):  # {{{
             if cdata:
                 cache.set_cover({bid:cdata for bid in self.ids if bid != book_id})
 
-        # Formats
-        if args.do_remove_format:
-            cache.remove_formats({bid:(args.remove_format,) for bid in self.ids})
-
         if args.restore_original:
             for book_id in self.ids:
                 formats = cache.formats(book_id)
@@ -307,7 +306,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
                     }
 
     s_r_match_modes = [_('Character match'),
-                            _('Regular Expression'),
+                            _('Regular expression'),
                       ]
 
     s_r_replace_modes = [_('Replace field'),
@@ -340,11 +339,6 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
 
         self.initialize_combos()
 
-        for f in sorted(self.db.all_formats()):
-            self.remove_format.addItem(f)
-
-        self.remove_format.setCurrentIndex(-1)
-
         self.series.currentIndexChanged[int].connect(self.series_changed)
         self.rating.currentIndexChanged.connect(lambda:self.apply_rating.setChecked(True))
         self.series.editTextChanged.connect(self.series_changed)
@@ -449,6 +443,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
     def prepare_search_and_replace(self):
         self.search_for.initialize('bulk_edit_search_for')
         self.replace_with.initialize('bulk_edit_replace_with')
+        self.s_r_template.setLineEdit(TemplateLineEditor(self.s_r_template))
         self.s_r_template.initialize('bulk_edit_template')
         self.test_text.initialize('bulk_edit_test_test')
         self.all_fields = ['']
@@ -520,7 +515,7 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
 
         self.regexp_heading = _(
                  'In regular expression mode, the search text is an '
-                 'arbitrary python-compatible regular expression. The '
+                 'arbitrary Python-compatible regular expression. The '
                  'replacement text can contain backreferences to parenthesized '
                  'expressions in the pattern. The search is not anchored, '
                  'and can match and replace multiple times on the same string. '
@@ -529,8 +524,8 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
                  'The destination box specifies the field where the result after '
                  'matching and replacement is to be assigned. You can replace '
                  'the text in the field, or prepend or append the matched text. '
-                 'See <a href="http://docs.python.org/library/re.html"> '
-                 'this reference</a> for more information on python\'s regular '
+                 'See <a href="https://docs.python.org/library/re.html">'
+                 'this reference</a> for more information on Python\'s regular '
                  'expressions, and in particular the \'sub\' function.'
                  )
 
@@ -1018,13 +1013,12 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         do_series_restart = self.series_numbering_restarts.isChecked()
         series_start_value = self.series_start_number.value()
         series_increment = self.series_increment.value()
-        do_remove_format = self.remove_format.currentIndex() > -1
-        remove_format = unicode(self.remove_format.currentText())
         do_swap_ta = self.swap_title_and_author.isChecked()
         do_remove_conv = self.remove_conversion_settings.isChecked()
         do_auto_author = self.auto_author_sort.isChecked()
         do_title_case = self.change_title_to_title_case.isChecked()
         do_title_sort = self.update_title_sort.isChecked()
+        read_file_metadata = self.read_file_metadata.isChecked()
         clear_languages = self.clear_languages.isChecked()
         restore_original = self.restore_original.isChecked()
         languages = self.languages.lang_codes
@@ -1046,12 +1040,17 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         elif self.cover_clone.isChecked():
             cover_action = 'clone'
 
-        args = Settings(remove_all, remove, add, au, aus, do_aus, rating, pub, do_series,
-                do_autonumber, do_remove_format, remove_format, do_swap_ta,
-                do_remove_conv, do_auto_author, series, do_series_restart,
-                series_start_value, series_increment, do_title_case, cover_action, clear_series, clear_pub,
-                pubdate, adddate, do_title_sort, languages, clear_languages,
-                restore_original, self.comments, self.generate_cover_settings)
+        args = Settings(
+            remove_all, remove, add, au, aus, do_aus, rating, pub, do_series,
+            do_autonumber, do_swap_ta, do_remove_conv, do_auto_author, series,
+            do_series_restart, series_start_value, series_increment,
+            do_title_case, cover_action, clear_series, clear_pub, pubdate,
+            adddate, do_title_sort, languages, clear_languages,
+            restore_original, self.comments, self.generate_cover_settings,
+            read_file_metadata)
+        if DEBUG:
+            print('Running bulk metadata operation with settings:')
+            print(args)
 
         self.set_field_calls = defaultdict(dict)
         bb = MyBlockingBusy(args, self.ids, self.db, self.refresh_books,
@@ -1228,5 +1227,3 @@ class MetadataBulkDialog(QDialog, Ui_MetadataBulkDialog):
         self.results_count.setValue(999)
         self.starting_from.setValue(1)
         self.multiple_separator.setText(" ::: ")
-
-
