@@ -29,7 +29,7 @@ from calibre.constants import DEBUG
 from calibre.ebooks.oeb.polish.report import (
     gather_data, CSSEntry, CSSFileMatch, MatchLocation, ClassEntry,
     ClassFileMatch, ClassElement, CSSRule, LinkLocation)
-from calibre.gui2 import error_dialog, question_dialog, choose_save_file, open_url
+from calibre.gui2 import error_dialog, question_dialog, choose_save_file, open_url, secure_web_page
 from calibre.gui2.tweak_book import current_container, tprefs, dictionaries
 from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.gui2.progress_indicator import ProgressIndicator
@@ -40,12 +40,14 @@ from calibre.utils.localization import calibre_langcode_to_name, canonicalize_la
 
 ROOT = QModelIndex()
 
+
 def read_state(name, default=None):
     data = tprefs.get('reports-ui-state')
     if data is None:
         tprefs['reports-ui-state'] = data = {}
     return data.get(name, default)
 
+
 def save_state(name, val):
     data = tprefs.get('reports-ui-state')
     if isinstance(val, QByteArray):
@@ -54,8 +56,10 @@ def save_state(name, val):
         tprefs['reports-ui-state'] = data = {}
     data[name] = val
 
+
 SORT_ROLE = Qt.UserRole + 1
 
+
 class ProxyModel(QSortFilterProxyModel):
 
     def __init__(self, parent=None):
@@ -81,6 +85,7 @@ class ProxyModel(QSortFilterProxyModel):
             return section + 1
         return QSortFilterProxyModel.headerData(self, section, orientation, role)
 
+
 class FileCollection(QAbstractTableModel):
 
     COLUMN_HEADERS = ()
@@ -107,9 +112,10 @@ class FileCollection(QAbstractTableModel):
     def location(self, index):
         try:
             return self.files[index.row()].name
-        except IndexError:
+        except (IndexError, AttributeError):
             pass
 
+
 class FilesView(QTableView):
 
     double_clicked = pyqtSignal(object)
@@ -213,6 +219,7 @@ class FilesView(QTableView):
 
 # Files {{{
 
+
 class FilesModel(FileCollection):
 
     COLUMN_HEADERS = (_('Folder'), _('Name'), _('Size (KB)'), _('Type'))
@@ -262,6 +269,7 @@ class FilesModel(FileCollection):
             if col == 3:
                 return self.CATEGORY_NAMES.get(entry.category)
 
+
 class FilesWidget(QWidget):
 
     edit_requested = pyqtSignal(object)
@@ -274,6 +282,7 @@ class FilesWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = FilesModel(self)
         self.files = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -307,6 +316,7 @@ class FilesWidget(QWidget):
 
 # Jump {{{
 
+
 def jump_to_location(loc):
     from calibre.gui2.tweak_book.boss import get_boss
     boss = get_boss()
@@ -327,6 +337,7 @@ def jump_to_location(loc):
         if loc.text_on_line is not None:
             editor.find(regex.compile(regex.escape(loc.text_on_line)))
 
+
 class Jump(object):
 
     def __init__(self):
@@ -341,10 +352,12 @@ class Jump(object):
             loc = locations[self.pos_map[key]]
             jump_to_location(loc)
 
+
 jump = Jump()  # }}}
 
 # Images {{{
 
+
 class ImagesDelegate(QStyledItemDelegate):
 
     MARGIN = 5
@@ -459,6 +472,7 @@ class ImagesWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = ImagesModel(self)
         self.files = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -496,9 +510,10 @@ class ImagesWidget(QWidget):
 
 # Links {{{
 
+
 class LinksModel(FileCollection):
 
-    COLUMN_HEADERS = ['✓ ', _('Source'), _('Source text'), _('Target'), _('Anchor'), _('Target text')]
+    COLUMN_HEADERS = ['✓', _('Source'), _('Source text'), _('Target'), _('Anchor'), _('Target text')]
 
     def __init__(self, parent=None):
         FileCollection.__init__(self, parent)
@@ -528,7 +543,7 @@ class LinksModel(FileCollection):
             except IndexError:
                 return None
             if col == 0:
-                return {True:'✓ ', False:'✗'}.get(link.ok)
+                return {True:'✓', False:'✗'}.get(link.ok)
             if col == 1:
                 return link.location.name
             if col == 2:
@@ -560,11 +575,13 @@ class LinksModel(FileCollection):
             except IndexError:
                 pass
 
+
 class WebView(QWebView):
 
     def sizeHint(self):
         return QSize(600, 200)
 
+
 class LinksWidget(QWidget):
 
     def __init__(self, parent=None):
@@ -576,6 +593,7 @@ class LinksWidget(QWidget):
         self.splitter = s = QSplitter(Qt.Vertical, self)
         l.addWidget(s)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = LinksModel(self)
         self.links = f = FilesView(m, self)
         f.DELETE_POSSIBLE = False
@@ -585,6 +603,9 @@ class LinksWidget(QWidget):
         s.addWidget(f)
         self.links.restore_table('links-table', sort_column=1)
         self.view = WebView(self)
+        secure_web_page(self.view.page())
+        self.setContextMenuPolicy(Qt.NoContextMenu)
+        self.view.setContextMenuPolicy(Qt.NoContextMenu)
         s.addWidget(self.view)
         self.ignore_current_change = False
         self.current_url = None
@@ -649,6 +670,7 @@ class LinksWidget(QWidget):
 
 # Words {{{
 
+
 class WordsModel(FileCollection):
 
     COLUMN_HEADERS = (_('Word'), _('Language'), _('Times used'))
@@ -660,11 +682,12 @@ class WordsModel(FileCollection):
         self.total_size = len({entry.locale for entry in self.files})
         psk = numeric_sort_key
         lsk_cache = {}
+
         def locale_sort_key(loc):
             try:
                 return lsk_cache[loc]
             except KeyError:
-                lsk_cache[loc] = (psk(calibre_langcode_to_name(canonicalize_lang(loc[0]))), psk(loc[1] or ''))
+                lsk_cache[loc] = psk(calibre_langcode_to_name(canonicalize_lang(loc[0])) + (loc[1] or ''))
             return lsk_cache[loc]
 
         self.sort_keys = tuple((psk(entry.word), locale_sort_key(entry.locale), len(entry.usage)) for entry in self.files)
@@ -700,6 +723,7 @@ class WordsModel(FileCollection):
     def location(self, index):
         return None
 
+
 class WordsWidget(QWidget):
 
     def __init__(self, parent=None):
@@ -709,6 +733,7 @@ class WordsWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = WordsModel(self)
         self.words = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -742,6 +767,7 @@ class WordsWidget(QWidget):
 
 # Characters {{{
 
+
 class CharsModel(FileCollection):
 
     COLUMN_HEADERS = (_('Character'), _('Name'), _('Codepoint'), _('Times used'))
@@ -786,6 +812,7 @@ class CharsModel(FileCollection):
     def location(self, index):
         return None
 
+
 class CharsWidget(QWidget):
 
     def __init__(self, parent=None):
@@ -795,6 +822,7 @@ class CharsWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = CharsModel(self)
         self.chars = f = FilesView(m, self)
         self.to_csv = f.to_csv
@@ -855,6 +883,7 @@ class CharsWidget(QWidget):
 
 # CSS {{{
 
+
 class CSSRulesModel(QAbstractItemModel):
 
     def __init__(self, parent):
@@ -957,6 +986,7 @@ class CSSRulesModel(QAbstractItemModel):
         self.build_maps()
         self.endResetModel()
 
+
 class CSSProxyModel(QSortFilterProxyModel):
 
     def __init__(self, parent=None):
@@ -977,6 +1007,7 @@ class CSSProxyModel(QSortFilterProxyModel):
             return True
         return primary_contains(self._filter_text, entry.rule.selector)
 
+
 class CSSWidget(QWidget):
 
     SETTING_PREFIX = 'css-'
@@ -997,6 +1028,7 @@ class CSSWidget(QWidget):
         self.filter_edit = e = QLineEdit(self)
         l.addWidget(e)
         e.setPlaceholderText(_('Filter'))
+        e.setClearButtonEnabled(True)
         self.model = m = self.MODEL(self)
         self.proxy = p = self.PROXY(self)
         p.setSourceModel(m)
@@ -1032,6 +1064,7 @@ class CSSWidget(QWidget):
     def sort_order(self):
         def fget(self):
             return [Qt.AscendingOrder, Qt.DescendingOrder][self._sort_order.currentIndex()]
+
         def fset(self, val):
             self._sort_order.setCurrentIndex({Qt.AscendingOrder:0}.get(val, 1))
         return property(fget=fget, fset=fset)
@@ -1100,6 +1133,7 @@ class CSSWidget(QWidget):
 
 # Classes {{{
 
+
 class ClassesModel(CSSRulesModel):
 
     def __init__(self, parent):
@@ -1177,6 +1211,7 @@ class ClassesModel(CSSRulesModel):
         self.build_maps()
         self.endResetModel()
 
+
 class ClassProxyModel(CSSProxyModel):
 
     def filterAcceptsRow(self, row, parent):
@@ -1188,6 +1223,7 @@ class ClassProxyModel(CSSProxyModel):
             return True
         return primary_contains(self._filter_text, entry.cls)
 
+
 class ClassesWidget(CSSWidget):
 
     SETTING_PREFIX = 'classes-'
@@ -1231,6 +1267,8 @@ class ClassesWidget(CSSWidget):
 # }}}
 
 # Wrapper UI {{{
+
+
 class ReportsWidget(QWidget):
 
     edit_requested = pyqtSignal(object)
@@ -1266,11 +1304,11 @@ class ReportsWidget(QWidget):
 
         self.css = c = CSSWidget(self)
         s.addWidget(c)
-        QListWidgetItem(_('Style Rules'), r)
+        QListWidgetItem(_('Style rules'), r)
 
         self.css = c = ClassesWidget(self)
         s.addWidget(c)
-        QListWidgetItem(_('Style Classes'), r)
+        QListWidgetItem(_('Style classes'), r)
 
         self.chars = c = CharsWidget(self)
         s.addWidget(c)
@@ -1320,6 +1358,7 @@ class ReportsWidget(QWidget):
             with open(fname, 'wb') as f:
                 f.write(data)
 
+
 class Reports(Dialog):
 
     data_gathered = pyqtSignal(object, object)
@@ -1413,6 +1452,7 @@ class Reports(Dialog):
         Dialog.reject(self)
 # }}}
 
+
 if __name__ == '__main__':
     from calibre.gui2 import Application
     import sys
